Package: arg
Type: Package
Title: Clean and Simple Argument Checking
Version: 0.2.1
Authors@R: person("Noah", "Greifer", role = c("aut", "cre"),
         email = "noah.greifer@gmail.com",
	       comment = c(ORCID="0000-0003-3067-7154"))
Description: Checks function arguments, ideally for use in R packages. Uses a simple interface and produces clean, informative error messages using 'cli'.
Depends: R (>= 4.1.0)
Imports: rlang (>= 1.2.0), cli (>= 3.6.4), stats, utils
License: GPL (>= 2)
Encoding: UTF-8
URL: https://ngreifer.github.io/arg/, https://github.com/ngreifer/arg
BugReports: https://github.com/ngreifer/arg/issues
Config/roxygen2/version: 8.0.0
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-29 16:46:22 UTC; NoahGreifer
Author: Noah Greifer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3067-7154>)
Maintainer: Noah Greifer <noah.greifer@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-30 05:10:13 UTC
Built: R 4.5.2; ; 2026-07-30 06:30:45 UTC; unix
