# Generated by roxygen2: do not edit by hand

export(arg_all_NA)
export(arg_and)
export(arg_array)
export(arg_atomic)
export(arg_between)
export(arg_character)
export(arg_colnamed)
export(arg_color)
export(arg_colors)
export(arg_colour)
export(arg_colours)
export(arg_cor)
export(arg_count)
export(arg_counts)
export(arg_cov)
export(arg_data)
export(arg_data.frame)
export(arg_distance)
export(arg_dots_not_supplied)
export(arg_dots_supplied)
export(arg_element)
export(arg_env)
export(arg_equal)
export(arg_factor)
export(arg_flag)
export(arg_formula)
export(arg_function)
export(arg_gt)
export(arg_gte)
export(arg_index)
export(arg_indices)
export(arg_is)
export(arg_is_NA)
export(arg_is_not)
export(arg_length)
export(arg_list)
export(arg_logical)
export(arg_lt)
export(arg_lte)
export(arg_matrix)
export(arg_name)
export(arg_named)
export(arg_names)
export(arg_no_NA)
export(arg_non_null)
export(arg_not_element)
export(arg_not_equal)
export(arg_null)
export(arg_number)
export(arg_numeric)
export(arg_or)
export(arg_ordered)
export(arg_string)
export(arg_supplied)
export(arg_symmetric)
export(arg_unique)
export(arg_vector)
export(arg_whole_number)
export(arg_whole_numeric)
export(err)
export(match_arg)
export(msg)
export(when_not_null)
export(when_supplied)
export(wrn)
importFrom(stats,setNames)
importFrom(utils,capture.output)
