# arg

[![Actions Status](https://github.com/DoumanAsh/arg.rs/workflows/Rust/badge.svg)](https://github.com/DoumanAsh/arg.rs/actions)
[![Crates.io](https://img.shields.io/crates/v/arg.svg)](https://crates.io/crates/arg)
[![Documentation](https://docs.rs/arg/badge.svg)](https://docs.rs/crate/arg/)

Very simple code generator for command line argument parsing.

No dependencies except proc macro (with its dependencies) used at build time.

## Features

- `std` - Enables utilities that require `std` library.

## Debug

Set env variable `ARG_RS_PRINT_PARSER` to any value except `0` or `false` to print parser result
