% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg-package.R
\docType{package}
\name{arg-package}
\alias{arg}
\alias{arg-package}
\title{arg: Clean and Simple Argument Checking}
\description{
Checks function arguments, ideally for use in R packages. Uses a simple interface and produces clean, informative error messages using 'cli'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ngreifer.github.io/arg/}
  \item \url{https://github.com/ngreifer/arg}
  \item Report bugs at \url{https://github.com/ngreifer/arg/issues}
}

}
\author{
\strong{Maintainer}: Noah Greifer \email{noah.greifer@gmail.com} (\href{https://orcid.org/0000-0003-3067-7154}{ORCID})

Authors:
\itemize{
  \item Noah Greifer \email{noah.greifer@gmail.com} (\href{https://orcid.org/0000-0003-3067-7154}{ORCID})
}

}
\keyword{internal}
